%==================================================%
% Name:                                            %
%   write_cnv_ncdf_G06.m                                 %
% Purpose:                                         %
%   The purpose of this program is converting  	   %
%   CTD data into a NetCDF data Document           %
% Needed Subprograms:(function at end of program)  %
%   year2sec1970.m, write_nc_G06.m                 %
%==================================================%

clear all; 
close all;
% define global variables to pass to function
global featureType title summary sea_name program_name project_name id_ori country
global creator_name creator_url creator_email creator_phone creator_inst institution
global keywords_vocab keywords instruments Shipname source value platform platform_vocab
global station pub_name pub_email pub_phone pub_url pub_inst ref_time

%-------------------------------------------------------------------------------------------------------
% The user should enter the common metadata that will apply to all of the CTD data from this cruise.
%-------------------------------------------------------------------------------------------------------

% Data information

featureType = 'profile';
title = 'Gulf Integrated Spill Research Consortium( GISR) Cruise G06 Mooring Removal Cruise CTD Profile.';
summary = 'Five Moorings were removed on this cruise. These are the associated CTD profiles.';
sea_name = 'Gulf of Mexico';
program_name = 'The Gulf of Mexico Research Initiative (GOMRI)';
project_name = 'Gulf Integrated Spill Research Consortium( GISR)';
id_ori = 'R/V_Pelican_Cruise_PE1426';
country = 'USA';
institution = 'Texas A&M University: Dept. of Oceanography';
creator_name = 'Dr. Steven DiMarco';
creator_url = 'http://ocean.tamu.edu/people/faculty/dimarcosteve.html';
creator_email= 'sdimarco@tamu.edu';
creator_phone = '+1-979-862-4168';
creator_inst = 'Texas A&M University';
pub_name = 'Dr. Matthew K. Howard';
pub_email = 'mkhoward@tamu.edu';
pub_phone = '+1-979-862-4169';
pub_url = 'http://ocean.tamu.edu/people/researchpersonnel/howardmatthewk.html';
pub_inst ='Texas A&M University';
keywords = 'Gulf of Mexico, CTD, Water Column data';
keywords_vocab = 'GCMD Science Keywords Version 8.1';
instruments ='Seabird SBE9,Chelsea Aqua 3, WET Labs C-Star, LICOR Biospherical PAR';
Shipname = 'R/V Pelican';
cruise ='';
platform = 'R/V Pelican';
platform_vocab = 'http:;;www.nodc.noaa.gov/General/NODC-Archive/platformlist.txt';
source = 'CTD profiles from a ship ';
value = '';
fillvalue = 999.;

filedir  = '/Users/marion/GOMRI/GRIIDC/GISR_GRIIDC/'
filedir2 = '/Users/marion/GOMRI/GRIIDC/GISR_GRIIDC/'
%}
file_id = dir(fullfile(filedir,'G06L01S01.cnv'));


mona = {'Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec'};
monu = {'01','02','03','04','05','06','07','08','09','10','11','12'};
%
for m = 1:length(file_id) 
%    
% initialize some variables and paths
%
    filein = [filedir,file_id(m).name];    % file path and name
    filid  = file_id(m).name;
    fprintf(1,'%s\n',filein);
    FILENAME = [filedir2,regexprep((file_id(m).name),'.cnv',''),'_2.nc']
    fid = fopen(filein, 'r');
    
    sip = 370    % minimal number of lines/rows until data variables start (after *END* ) 
    l00 = textscan(fid,'%s',sip,'Delimiter','\n');
    l01 =char(l00{:}(:))
    la0 =''; lo0 = ''; lt0 =''; lap = ''; la1 =''; la2 =''; la3 =''; lim = 0;    
    for i=1:sip      
      if(strcmp(l01(i,1:15),'* NMEA Latitude') == 1)
         la0 = l01(i,:);
      end
      if(strcmp(l01(i,1:16),'* NMEA Longitude') == 1) 
         lo0 = l01(i,:); 
      end
      if(strcmp(l01(i,1:17),'* NMEA UTC (Time)') == 1)
         lt0 = l01(i,:); lim = 1; % UTC or local time
      end
      if(strcmp(l01(i,1:11),'** PROJECT:') == 1)
          lap = l01(i,:);
      else
          lap = '';
      end      
      if(strcmp(l01(i,1:10),'** CRUISE:') == 1)
          la1 = l01(i,:);
      else
          la1 = '';
      end
      if((strcmp(l01(i,1:12),'** PLATFORM:') == 1) || (strcmp(l01(i,1:8),'** Ship:') == 1))
          la2 = l01(i,:);
      else
          la2 = '';
      end
      if((strcmp(l01(i,1:16),'** STATION NAME:') == 1) || (strcmp(l01(i,1:11),'** Station:') == 1))
          la3 = l01(i,:);
      else
          la3 = '';
      end      
      if(strcmp(l01(i,1:7),'# nquan') == 1 )
         la5 = l01(i,:); 
      end
      if(strcmp(l01(i,1:9),'# nvalues') == 1 )
          la6 = l01(i,:);
      end
      if(strcmp(l01(i,1:19),'# units = specified') == 1 )        
      end
      if(strcmp(l01(i,1:5),'*END*') == 1 )
          la7 = l01(i,:); 
          iend = i;
      end
%        
    end   

    k01       = strfind(la0,'=');
    k02       = strfind(la0,' ');
    ia        = find(k02 >= k01);
    lade      = str2num(la0(k02(ia(1)):k02(ia(2)))); 
    if (isempty(ia(3)) ~= 1)
      lami      = str2num(la0(k02(ia(2))+1:k02(ia(3))));
    else
      lami      = str2num(la0(k02(ia(2))+1:end));
    end
    lat1      = lade+(lami/60);
    
    k03       = strfind(lo0,'=');
    k04       = strfind(lo0,' ');
    ib        = find(k04 >= k03);
    lode      = str2num(lo0(k04(ib(1)):k04(ib(2))));
    if (isempty(ib(3)) ~= 1)
      lomi      = str2num(lo0(k04(ib(2))+1:k04(ib(3))));
    else
      lomi      = str2num(lo0(k04(ib(2))+1:end));  
    end
    lon1      = (lode+(lomi/60))*(-1);
%    if (~isempty(la2) | la2 ~= ' ')
    if isempty(Shipname)
    k05      = strfind(la2,':');
    k06     = strfind(la2,' ');
    k0       = find(k06 >= k05);
    Shipname = la2(k06(k0(1))+1:k06(k0(2))-1);
%    Shipname = [Shipname(1:3),'_',Shipname(5:9),'_',Shipname(11:13)]
    end
    if ~isempty(la1)
    k07      = strfind(la1,':');
    k08     = strfind(la1,' ');
    ci       = find(k08 >= k07);    
    cruise  = [la1(k08(2)+1:k08(3)-1),'_',la1(k08(3)+1:k08(4)-1)];
    end
%    if(isempty(la1)) cruise  = '', end
    if ~isempty(la3)
%    if (~isempty(la3) | la3 ~= ' ')
     ka2      = strfind(la3,':'); 
     ka3       = strfind(la3,' ');
     ic        = find(ka3 >= ka2);  
     if (length(ic) >= 3 && isempty(ic(3)) ~= 1)
      station = [la3(ka3(ic(1))+1:ka3(ic(2))-1)];    
%    station = [la3(ka3(ic(1))+1:ka3(ic(2))-1),'_',la3(ka3(ic(2))+1:ka3(ic(3))-1),...
%        '_',la3(ka3(ic(3))+1:end)];
     elseif(length(ic) >= 2 && isempty(ic(2)) ~= 1) 
      station = [la3(ka3(ic(1))+1:ka3(ic(2))-1),'_',la3(ka3(ic(2))+1:end)];
     elseif( isempty(ic) == 1)
        station = '';
     else
        station = [la3(ka3(ic(1))+1:end)];
     end
    else 
       station = filid(9:end-4);
    end
    
    event   = '';
    ld1     = strfind(lt0,'=');
    dat     = lt0(ld1+1:end);
    ld2     = strfind(lt0,' ');
    ld3     = strfind(lt0,':');
    id      = find(ld2 >= ld1);
    mon1    = lt0(ld2(id(1))+1:ld2(id(2))-1);
    x       = strmatch(char(mon1),mona);
    mon     = str2num(char(monu(x)));
    tag     = str2num(lt0(ld2(id(2))+1:ld2(id(3))-1));
    if (isempty(id(4)) ~= 1)
      jahr    = str2num(lt0(ld2(id(3))+1:ld2(id(4))-1));
      tim = lt0(ld3(1)-2:end);
    else
        jahr    = str2num(lt0(ld2(id(3))+1:end));
        tim = '';
    end
   
    if( lim ~= 1)
      hr = str2num(tim(1:2))+6; % local time EST + 6
      if( hr > 23)
          tag = tag + 1;
          hr = hr-24;
      end
    else
      hr = str2num(tim(1:2));
    end
    mn = str2num(tim(4:5)); 
    sc = str2num(tim(7:8));
    dat1 = [num2str(jahr),'-',num2str(mon),'-',num2str(tag)];
    tim1 = [num2str(hr),':',num2str(mn),':',num2str(sc)];
 
    timst  = regexprep(strcat(dat1,'!',tim1),'!',' '); % start time
    timend = regexprep(strcat(dat1,'!',tim1),'!',' '); % end time
    
%- Yearday to seconds since 1970-01-01 00:00:00
    ref_time = 'Seconds since 1970-01-01 00:00:00';
    [nusec ] =year2sec1970(jahr,mon,tag,hr,mn,sc);
    num_time = size(nusec,2);       
    if(strcmp(event,'Unknown') == 1), event = ''; end; 
    if(exist('cruise','var') ~= 1 || isempty(cruise) == 1)
       if(exist('station','var') ~= 1 || isempty(station) == 1)
          profile_name = [id_ori,'-',dat1];
       else
          profile_name = [id_ori,'-',station];
       end
    elseif(strcmp(id_ori,cruise) == 1)
           if(exist('station','var') ~= 1 || isempty(station) == 1)
                profile_name = [id_ori,'-',dat1];
           else
                profile_name = [id_ori,'-',station];
           end
    elseif(strcmp(id_ori,cruise) ~= 1 && isempty(cruise) ~= 1)
        if(exist('station','var') ~= 1 || isempty(station) == 1)
                profile_name = [id_ori,'-',cruise,'-',dat1];
        else
                profile_name = [id_ori,'-',cruise,'-',station];
        end
    end

    frewind(fid)
    l02    = textscan(fid,'%s ',iend,'Delimiter','\n'); 
    li = 0;
    while ~feof(fid)

      l03 = textscan(fid,...          
      '%f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f',1); 
%                   |              |              |              |              |              |
      if ~isempty(l03{2})
        
       li = li + 1;
%{                  
# name 0 = prDM: Pressure, Digiquartz [db]
# name 1 = t090C: Temperature [ITS-90, deg C]
# name 2 = t190C: Temperature, 2 [ITS-90, deg C]
# name 3 = c0S/m: Conductivity [S/m]
# name 4 = c1S/m: Conductivity, 2 [S/m]
# name 5 = CStarAt0: Beam Attenuation, WET Labs C-Star [1/m]
# name 6 = CStarTr0: Beam Transmission, WET Labs C-Star [%]
# name 7 = spar: SPAR/Surface Irradiance
# name 8 = par: PAR/Irradiance, Biospherical/Licor
# name 9 = cpar: CPAR/Corrected Irradiance [%]
# name 10 = flC: Fluorescence, Chelsea Aqua 3 Chl Con [ug/l]
# name 11 = wetCDOM: Fluorescence, WET Labs CDOM [mg/m^3]
# name 12 = sbeox0V: Oxygen raw, SBE 43 [V]
# name 13 = sbeox1V: Oxygen raw, SBE 43, 2 [V]
# name 14 = sbeox0ML/L: Oxygen, SBE 43 [ml/l]
# name 15 = sbeox1ML/L: Oxygen, SBE 43, 2 [ml/l]
# name 16 = altM: Altimeter [m]
# name 17 = depSM: Depth [salt water, m], lat = 28.2498
# name 18 = sigma-00: Density [sigma-theta, kg/m^3]
# name 19 = sigma-11: Density, 2 [sigma-theta, kg/m^3]
# name 20 = potemp090C: Potential Temperature [ITS-90, deg C]
# name 21 = potemp190C: Potential Temperature, 2 [ITS-90, deg C]
# name 22 = sal00: Salinity, Practical [PSU]
# name 23 = sal11: Salinity, Practical, 2 [PSU]
# name 24 = flag: flag
%}
      pres(li)    = l03{1};     unpr  = 'dbar';      
      temp1(li)    = l03{2};     unte  = 'degrees Celsius';
      temp2(li)    = l03{3};     unte  = 'degrees Celsius';
      cond1(li)   = l03{4};     unc1  = 'S/m';
      cond2(li)   = l03{5};     unc1  = 'S/m';
      CSbat(li)   = l03{6};     unba  = '1/m';
      xmiss(li)   = l03{7};     unta  = '%';
      spar(li)    = l03{8};     unpa  ='watts per square meter'; %unpa = 'uE/m^2/s';
      par(li)     = l03{9};     unpa  ='microEinstiens/meter^2 sec'; %unpa = 'uE/m^2/s';
      cpar(li)    = l03{10};    unpa  ='%'; 
      flC(li)     = l03{11};    unfl  = 'ug/l';
      wetCDOM(li) = l03{12};    unfl  = 'mg/m^3';
      oxyr1(li)   = l03{13};    uro1  = 'Volt'; %'umol/kg';
      oxyr2(li)   = l03{14};    uro2  = 'Volt';
      oxy1(li)    = l03{15};    uox1  = 'ml/l'; %'umol/kg';
      oxy2(li)    = l03{16};    uox2  = 'ml/l';
      alt(li)     = l03{17};    unal  = 'm'; 
      dep(li)     = l03{18};    unde  = 'm';
      dens1(li)   = l03{19};    unsi  = 'kg/m^3';
      dens2(li)   = l03{20};    unsi  = 'kg/m^3';
      potemp1(li) = l03{21};    unt   = 'degrees Celsius';
      potemp2(li) = l03{22};    unt   = 'degrees Celsius';   
      sal1(li)    = l03{23};    unsa  = 'psu';
      sal2(li)    = l03{24};    unsa  = 'psu';           
      end
    end
     
    li
    nbin = length(dep);
    
    zlen = nbin
    lat =lat1
    lon =lon1
    if(length(lat) < 2)
        lalo = 0;
    else
        lalo =1;
    end
fclose(fid);
%- Write netdef file 
% Please add variables to function "write_nc_G06" if needed.

write_nc_G06(FILENAME,profile_name,nusec,lat,lon,dep,pres,temp1,temp2,cond1,cond2,cruise,...
    CSbat,xmiss,spar,par,cpar,flC,wetCDOM,oxyr1,oxyr2,oxy1,oxy2,alt,dens1,dens2,sal1,sal2,title,summary,...
    sea_name,program_name,project_name,id_ori,country,creator_name,creator_url,creator_email,creator_phone,...
    creator_inst,station,pub_name,pub_url,pub_email,pub_phone,pub_inst,keywords,keywords_vocab,instruments,...
    ref_time,Shipname,source,value,platform,platform_vocab,institution,featureType);


%clear variables for next run
clear l00 l01 filein fid FILENAME 
clear dat tim timst timend profile_name event
clear lat lon jahr mon tag hr mn sc nbin num_time nusec 
clear nscan dtim dep dep2 temp temp2 cond cond2 sal sal2 
clear oxy oxyr oxy1 oxy2 oxy_s dens1 chla dens dens2
clear nbin zlen flECO wetCDOM fluor tur  par bat xmiss alt 
clear svcm1 svcm2 svcm3 pres potemp potemp2 sigmat sigmat2
clear rho  oxy2 spar CSbat cpar
end
%end

clearvars -global
disp '*******  end of write_csv_ncdf.m program  *******'
%{
% function year2sec1970.m
%*******************************************************************************************
function newsec = year2sec1970(year,mo,da,ho,mi,se)

%- digital format change
format longg
% Define variables

%%{ example:
year = 1922;
mo = 02;
da = 1;
ho = 12;
mi = 0;
se = 0;
%%}


year1 = 1970;
year2 = year;
dmm = 24*60*60;
ia = 0;
yearsum = zeros((year2-year1+1),1)
mon = [31 28 31 30 31 30 31 31 30 31 30 31];
%
for i = year1:year2-1

%- Days in each month, account for leap years
   ia = ia + 1;
  
   if ( mod(i,4) == 0 & mod(i,400) == 0)   %leap year
       mon(2) = 29;
       days_in_year = 366
   else 
      mon(2) = 28;
      days_in_year = 365
   end     
%
%
   try 
      yearsum(ia) = days_in_year*dmm   %year != 1970
   catch
      if year1 == year2
	  year1
	     if month ~= 1
		    yearsum(ia) = sum(mon(1:mo-1))*dmm + (da-1)*dmm + (ho*3600) + (mi*60) + se
		 else
		    yearsum(ia) = (da-1) * dmm + (ho*3600)+ (mi*60) + se;
		 end
	  end
   end
%
%
end
%
yearsum(ia+1) = sum(mon(1:mo-1))*dmm +(da-1)*dmm +ho*60*60 +mi*60+ se;

newsec = sum(yearsum);

%*****************************************************************************
%}


